define(function (require, exports, module) {

    var $ = require('jquery');
    var Class = require('mod/class');
    var EventBase = require('mod/eventBase');
    var PublicDefaults = require('mod/cascadePublicDefaults');
    var CascadeItem = require('mod/cascadeItem');

    /**
     * PublicDefaults的作用见CascadeItem组件内的注释
     */
    var DEFAULTS = $.extend({}, PublicDefaults, {
        $elements: undefined, //级联项jq对象的数组，元素在数据中的顺序代表级联的先后顺序
        valueSeparator: ',', //获取所有级联项的值时使用的分隔符，如果是英文逗号，返回的值形如 北京市,区,朝阳区
        values: '', //用valueSeparator分隔的字符串，表示初始时各个select的值
        onChanged: $.noop //当任意级联项的值发生改变的时候会触发这个事件
    });

    var CascadeView = Class({
        instanceMembers: {
            init: function (options) {
                //通过this.base调用父类EventBase的init方法
                this.base();

                var opts = this.options = this.getOptions(options),
                    items = this.items = [],
                    that = this,
                    $elements = opts.$elements,
                    values = opts.values.split(opts.valueSeparator);

                this.on('changed.cascadeView', $.proxy(opts.onChanged, this));

                $elements && $elements.each(function (i) {
                    var $el = $(this);

                    //实例化CascadeItem组件，并把每个实例的prevItem属性指向前一个实例
                    //第一个prevItem属性设置为undefined
                    var cascadeItem = new CascadeItem($el, $.extend(that.getItemOptions(), {
                        prevItem: i == 0 ? undefined : items[i - 1],
                        value: $.trim(values[i])
                    }));
                    items.push(cascadeItem);

                    //每个级联项实例发生改变都会触发CascadeView组件的changed事件
                    //外部可在这个回调内处理业务逻辑
                    //比如将所有级联项的值设置到一个隐藏域里面，用于表单提交
                    cascadeItem.on('changed.cascadeItem', function () {
                        that.trigger('changed.cascadeView', that.getValue());
                    });
                });

                //初始化完成自动加载第一个级联项
                items.length && items[0].load();
            },
            getOptions: function (options) {
                return $.extend({}, this.getDefaults(), options);
            },
            getDefaults: function () {
                return DEFAULTS;
            },
            getItemOptions: function () {
                var opts = {}, _options = this.options;
                for (var i in PublicDefaults) {
                    if (PublicDefaults.hasOwnProperty(i) && i in _options) {
                        opts[i] = _options[i];
                    }
                }
                return opts;
            },
            //获取所有级联项的值，是一个用valueSeparator分隔的字符串
            //为空的级联项的值不会返回
            getValue: function () {
                var value = [];
                this.items.forEach(function (item) {
                    var val = $.trim(item.getValue());
                    val != '' && value.push(val);
                });
                return value.join(this.options.valueSeparator);
            }
        },
        extend: EventBase
    });

    return CascadeView;
});